#include <stdio.h>
#include <stdlib.h> 
#include <time.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <iostream>
#include <fstream>
#include <string.h>
#include <algorithm>
#include <vector>
#include <set>
#include <map>
#include "model.h"
#include "fpmath.h"
#include "control.h"
#include "gsl/gsl_linalg.h"
#include "gsl/gsl_permutation.h"
#include "gsl/gsl_permute.h"
#include "gsl/gsl_cdf.h"
#include <gsl/gsl_math.h>
#include <gsl/gsl_eigen.h>
#include <sys/time.h>
#include <pthread.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_roots.h>

#include "demo_fn.h"
#include "demo_fn.cpp"

typedef vector<vector<double> > Mat;
#include <sstream>
using namespace std;

#if defined (MPI_ENABLED)
#include "mpi.h"
#endif

ModelnData::ModelnData(void)
{
	
	fnOutput.assign("\0");
}


ModelnData::~ModelnData()
{
	vGin.clear();
	vPin.clear(); 
}


void ModelnData::GetInverse(gsl_matrix *A, gsl_matrix *inverse)
{
    int n = A->size1;
    gsl_matrix *tmpA = gsl_matrix_alloc(n, n);
    gsl_matrix_memcpy(tmpA, A);
    gsl_permutation *p = gsl_permutation_alloc(n);
    int sign = 0;
    gsl_linalg_LU_decomp(tmpA, p, &sign);
    gsl_linalg_LU_invert(tmpA, p, inverse);
    gsl_permutation_free(p);
    gsl_matrix_free(tmpA);
}//a function to get inverse matrix

double ModelnData::get_det(gsl_matrix *A)
{
    double det=0.0;
    int n=A->size1;
    gsl_permutation *p=gsl_permutation_calloc(n);
    gsl_matrix *tmpA = gsl_matrix_calloc(n, n);
    int signum;
    gsl_matrix_memcpy(tmpA,A);
    gsl_linalg_LU_decomp(tmpA,p,&signum);
    det=gsl_linalg_LU_det(tmpA,signum);
    gsl_permutation_free(p);
    gsl_matrix_free(tmpA);
    return det;
}//a function to get det_value

double ModelnData::lgbf_mm(gsl_matrix *m1s, gsl_matrix *m1y, double sa,int pm,int dd,int pp)
{
if(pm == 0) pm = dd -1;
    double lbf;
    if(sa==0){
		int status;
	    int iter=0;
	    int max_iter=100;
		const gsl_root_fsolver_type *T= gsl_root_fsolver_brent;
		gsl_root_fsolver *s=gsl_root_fsolver_alloc (T);
		double r = 0;
		double x_lo =0.00001, x_hi = 1;
		struct my1_f_params * params = new struct my1_f_params;
        params->y=m1y;
		params->x=m1s;
		params->n=nn;
		params->d=dd;
		params->p=pp;
		double a=my1_f(x_lo,(void *)params);
		double b=my1_f(x_hi,(void *)params);
		int ctrl=1;
		while(b>0 && ctrl==1){
	         x_hi=x_hi*10;
	         b=my1_f(x_hi,(void *)params);
	         if(fabs(b)<0.00000001){
				  r=x_hi;
				  ctrl=0;
			 }
		 }
		while(a<0 && ctrl==1){
			x_lo=x_lo/10;
			a=my1_f(x_lo,(void *)params);
			if(fabs(a)<0.00000001){
				  r=x_lo;
				  ctrl=0;
			}
			if(fabs(x_lo)<0.00000001){
				  r=x_lo;
				  ctrl=0;
			}
		}
		if(a>0 && b<0 && r==0){
			 gsl_function F;
			 F.function = &my1_f;
			 F.params = params;
			 gsl_root_fsolver_set (s, &F, x_lo, x_hi);
			 do{
				   iter++;
				   status = gsl_root_fsolver_iterate (s);
				   r = gsl_root_fsolver_root (s);
				   x_lo = gsl_root_fsolver_x_lower (s);
				   x_hi = gsl_root_fsolver_x_upper (s);
				   status = gsl_root_test_interval (x_lo, x_hi,0, 0.001);
			   }
			   while (status == GSL_CONTINUE && iter<max_iter);
			   gsl_root_fsolver_free (s);
		 }
		//  cout<<"r="<<r<<endl;
		sa=sqrt(r);
		delete params;
	}


    gsl_matrix * m1i = gsl_matrix_alloc(pp, pp);
    gsl_matrix_set_identity(m1i);//m1i=I
    gsl_matrix * m1k = gsl_matrix_alloc(pp, pp);
    gsl_matrix_memcpy(m1k, m1i);
    gsl_matrix_scale(m1k,(1/sa)*(1/sa));//m1k=K
    double detk=get_det(m1k);//det(K)

    gsl_matrix * m1xx = gsl_matrix_alloc(pp, pp);
    gsl_matrix *m1yy=gsl_matrix_alloc(dd, dd);
    gsl_matrix *m11=gsl_matrix_alloc(dd,pp);
    gsl_matrix *m12=gsl_matrix_alloc(dd,nn);
    gsl_matrix *m13=gsl_matrix_alloc(dd,dd);
    gsl_matrix *m14=gsl_matrix_alloc(dd,dd);
    gsl_blas_dgemm(CblasTrans,CblasNoTrans,1,m1s,m1s,0,m1xx);//m1xx=crossprod(X)
    gsl_matrix_add(m1xx,m1k);//m1xx=XX+K
    double  detxxk=get_det(m1xx);//det(XX+K)

    gsl_blas_dgemm(CblasTrans,CblasNoTrans,1,m1y,m1y,0,m1yy);//m1yy=crossprod(Y)
    double detyy=get_det(m1yy);//det(YY)

    gsl_matrix * m1invxx = gsl_matrix_alloc(pp,pp);
    GetInverse(m1xx,m1invxx);//m1invxx=solve(XX+K)

    gsl_matrix *m15=gsl_matrix_alloc(dd,pp);
    gsl_blas_dgemm(CblasTrans,CblasNoTrans,1,m1y,m1s,0,m15);

    gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1,m15,m1invxx,0,m11);

    gsl_blas_dgemm(CblasNoTrans,CblasTrans,1,m11,m1s,0,m12);
    gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1,m12,m1y,0,m13);

    gsl_matrix_memcpy(m14, m1yy);
    gsl_matrix_sub(m14,m13);
    double detm14=get_det(m14);
    lbf=(0.5*dd*log(detk/detxxk)+0.5*(nn+pm)*log(detyy/detm14))/log(10);
    //p1= (0.5*dd*log(detk/detxxk)+0.5*(nn+pm)*log(1/detm14))/log(10);
   // p2= 0.5*(nn+pm)*log(1/detyy)/log(10);
    
   // gsl_matrix *m16=gsl_matrix_alloc(pp,nn);
   // gsl_blas_dgemm(CblasNoTrans,CblasTrans,1,m1invxx,m1s,0,m16);
   // gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1,m16,m1y,0,m1b);

    gsl_matrix_free(m1i);
    gsl_matrix_free(m1k);
    gsl_matrix_free(m1xx);
    gsl_matrix_free(m14);
    gsl_matrix_free(m1yy);
    gsl_matrix_free(m11);
    gsl_matrix_free(m12);
    gsl_matrix_free(m13);
    gsl_matrix_free(m1invxx);
    gsl_matrix_free(m15);
  //  gsl_matrix_free(m16);
    return lbf;
  
}

 double ModelnData::lgbf_v(gsl_matrix *m1y,gsl_matrix *evec1, gsl_vector *eval ,gsl_eigen_symmv_workspace *w, double sa,int pm,int dd,int pp){
 //   struct timeval beg0, end0 ;
	 
 //   struct timezone tz0;
 //   int getb0 = gettimeofday(&beg0, &tz0);
	 if(pm == 0) pm = dd -1;
     double lbf;
//   gsl_matrix_const_view QQ=gsl_matrix_const_submatrix (Q, 0, 0, nn,pp);
//	 gsl_matrix_const_view RR=gsl_matrix_const_submatrix (R, 0, 0, pp,pp);    //give up this way of getting QQ and RR for some surious reason

/*
	 for(int i=0;i<pp;i++){
		 for(int j=0;j<pp;j++){
			 cout<<gsl_matrix_get(tR,i,j)<<"\t";
		 }
		 cout<<endl;
	 }
	 for(int i=0;i<pp;i++){
		 for(int j=0;j<pp;j++){
			 cout<<gsl_matrix_get(m1ss,i,j)<<"\t";
		 }
		 cout<<endl;
     }
  */
/*
	 double ddm=0;

     for (int i = 0; i < nn; i++) {      
		 double r=0; 
         for (int j = 0; j < nn; j++){
			 double a= (gsl_matrix_get(m1ss,i,i)+gsl_matrix_get(m1ss,j,j) -2*gsl_matrix_get(m1ss,i,j))/nn; //S[i,i]+S[j,j]-2*S[i,j]
             gsl_matrix_set(m1dd, i, j,a);
             r=r+a;
             ddm=ddm+a;
         }     
         r=r/nn;
         gsl_vector_set(eval,i,r);//eval is not eigenvalues, just mean of rows here
     }
	 ddm=ddm/(nn*nn);
     for (int i = 0; i < nn; i++){
         for (int j = 0; j < nn; j++)
             gsl_matrix_set(m1ss, i, j,(gsl_matrix_get(m1dd,i,j)-gsl_vector_get(eval,i)-gsl_vector_get(eval,j)+ddm));
     }//m1ss is ddcenter here!
*/

//    struct timeval beg1, end1 ;
//    struct timezone tz;
//    int getb1 = gettimeofday(&beg1, &tz);

   
//     int gete1 = gettimeofday(&end1, &tz);
//     if(getb1 == 0 && gete1 == 0)

//     cout<<"Runtime of gsl_eigen_symmv   is: "<< (end1.tv_sec-beg1.tv_sec) + 1e-6 * (end1.tv_usec - beg1.tv_usec)<<endl;

     gsl_vector_scale(eval,1/gsl_vector_get(eval,0)); 

	 int k=0;

     while(k<pp && k<nn && fabs(gsl_vector_get(eval, k))>0.01){
		 k++;
	     //cout<<fabs(gsl_vector_get(eval, k))<<"\t";
             }
	
     
	 k=k-1;
    // cout<<endl;
   //  cout<<"k="<<k<<endl;   
	 if(k<=0)k=1;
     gsl_vector *m1eval = gsl_vector_alloc (k);//1
     gsl_matrix *m1evec = gsl_matrix_alloc (nn, k);//2
   
     if(sa==0){
        for (int i = 0; i <k; i++){
        gsl_vector_set(m1eval, i, gsl_vector_get(eval,i));

		 for(int j=0; j<nn; j++){
			 gsl_matrix_set(m1evec, j, i, gsl_matrix_get(evec1, j, i));

         }

     }}

     else{
for (int i = 0; i <k; i++){
        gsl_vector_set(m1eval, i, gsl_vector_get(eval,i)/gsl_vector_get(eval,0));
        double vm=0;
        for(int j=0; j<nn; j++){
                        vm=vm+gsl_matrix_get( evec1, j,i);
                }
        vm=vm/nn;
        double vs=0;

         for(int j=0; j<nn; j++){
                    vs=vs+(gsl_matrix_get( evec1, j,i)-vm)*(gsl_matrix_get( evec1, j,i)-vm);
                 }
         vs=vs/(nn-1);
         vs=sqrt(vs);
                 for(int j=0; j<nn; j++){
                         gsl_matrix_set(m1evec, j, i, (gsl_matrix_get(evec1, j, i)-vm)/vs);

         }

     }




}  
  
   int pp1=k;

    
    if(sa==0){
    int status;
	int iter=0;
	int max_iter=100;
    const gsl_root_fsolver_type *T= gsl_root_fsolver_brent;
	gsl_root_fsolver *s=gsl_root_fsolver_alloc (T);
	double r = 0;
	double x_lo =0.00001, x_hi = 1;
	struct my_f_params * params = new struct my_f_params;
    params->e= m1eval;
	params->y=m1y;
	params->u=m1evec;
	params->n=nn;
	params->d=dd;
	params->p=pp1;

    double a=my_f(x_lo,(void *)params);
    double b=my_f(x_hi,(void *)params);
	int ctrl=1;
    while(b>0 && ctrl==1){
        x_hi=x_hi*10;
		b=my_f(x_hi,(void *)params);

		if(fabs(b)<0.00000001){
        r=x_hi;         
		ctrl=0;
		}
	}

    while(a<0 && ctrl==1){
	   x_lo=x_lo/10;
	   a=my_f(x_lo,(void *)params);
	   if(fabs(a)<0.00000001){
	   r=x_lo;
	   ctrl=0;
	   }
       if(fabs(x_lo)<0.00000001){
        r=x_lo;
		ctrl=0;
	   }
	}


	if(a>0 && b<0 && r==0){

	gsl_function F;
	
	F.function = &my_f;
	F.params = params;
	gsl_root_fsolver_set (s, &F, x_lo, x_hi);
    do{
		                    
		iter++;
		status = gsl_root_fsolver_iterate (s);
		r = gsl_root_fsolver_root (s);
		x_lo = gsl_root_fsolver_x_lower (s);
		x_hi = gsl_root_fsolver_x_upper (s);
		status = gsl_root_test_interval (x_lo, x_hi,0, 0.001);
	}
	while (status == GSL_CONTINUE && iter<max_iter);
	gsl_root_fsolver_free (s);
	} 
  //  cout<<"r="<<r<<endl;
	sa=sqrt(r);
	delete params;
	}
gsl_matrix * m11 = gsl_matrix_alloc(pp1, pp1); //3
gsl_matrix_set_identity(m11);
gsl_matrix * m1k = gsl_matrix_calloc(pp1, pp1);//4

for (int i = 0; i < pp1; i++){
	  
	gsl_matrix_set(m1k, i, i,(1/gsl_vector_get(m1eval,i)/sa/sa));

}//m1k

double detk=get_det(m1k);//det(K)


gsl_matrix *m1yy=gsl_matrix_alloc(dd, dd);//   5
gsl_matrix *m12=gsl_matrix_alloc(dd,nn);//     6
gsl_matrix *m13=gsl_matrix_alloc(dd,dd);//     7

gsl_blas_dgemm(CblasTrans,CblasNoTrans,1,m1evec,m1evec,0,m11);//m11=crossprod(X)
/*for(int i=0;i<pp1;i++){
cout<<"test="<<gsl_matrix_get(m11,i,i)<<endl;;
}
*/
gsl_matrix_add(m11,m1k);//m11=XX+K
double  detxxk=get_det(m11);//det(XX+K)

gsl_blas_dgemm(CblasTrans,CblasNoTrans,1,m1y,m1y,0,m1yy);//m1yy=crossprod(Y)
double detyy=get_det(m1yy);//det(YY)

GetInverse(m11,m1k);//m1k=solve(XX+K)
gsl_matrix *m14=gsl_matrix_alloc(dd,pp1);  //    8
gsl_matrix *m15=gsl_matrix_alloc(dd,pp1);//     9
gsl_blas_dgemm(CblasTrans,CblasNoTrans,1,m1y,m1evec,0,m14);
gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1,m14,m1k,0,m15);
gsl_blas_dgemm(CblasNoTrans,CblasTrans,1,m15,m1evec,0,m12);
gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1,m12,m1y,0,m13);

gsl_matrix_sub(m1yy,m13);
double detm14=get_det(m1yy);
lbf=(0.5*dd*log(detk/detxxk)+0.5*(nn+pm)*log(detyy/detm14))/log(10);


gsl_matrix_free(m1evec);
gsl_vector_free(m1eval);

gsl_matrix_free(m1k);
gsl_matrix_free(m1yy);
gsl_matrix_free(m11);
gsl_matrix_free(m12);
gsl_matrix_free(m13);
gsl_matrix_free(m14);
gsl_matrix_free(m15);

//    int gete0 = gettimeofday(&end0, &tz0);
//        if(getb0 == 0 && gete0 == 0)

//     cout<<"Runtime of function lgbf_v() is: "<< (end0.tv_sec-beg0.tv_sec) + 1e-6 * (end0.tv_usec - beg0.tv_usec)<<endl<<"==================================================="<<endl;

    return lbf;
}



void ModelnData::a_genotype_thread( int dd, int pp, int thi , int dis){
	int bott=100*(thi);
	int top=0;
    if ((100*(thi+1))<(np/pp)){
		top=100*(thi+1);
	}
	else{
		 top=np/pp;
	}

	int pp1;
    fstream outfile;
    string sfn("output/");
	sfn.append(fnOutput);
	string s1("-thread");
	sfn.append(s1);
    char s[10];
    sprintf(s, "%d", thi);
    sfn.append(s);
	if (dis==0){
		sfn.append(".mm.txt");
		pp1=pp-1;
    }
	else{
		sfn.append(".dis.txt");
		pp1=pp;
	}
	outfile.open(sfn.c_str(), ios::out);
	if(!outfile.is_open()){
		cout << "can't open file ... " << endl;
		exit(0);
	 }
    gsl_matrix * m1s = gsl_matrix_alloc(nn, pp1);
	gsl_matrix * m1y = gsl_matrix_alloc(nn, dd);

	gsl_matrix * m1ss = gsl_matrix_alloc(pp, pp);//1.m1ss=S
	gsl_vector *eval = gsl_vector_alloc (pp);//6
	gsl_matrix *evec = gsl_matrix_alloc (pp, pp);//7
	gsl_eigen_symmv_workspace * w = gsl_eigen_symmv_alloc (pp);
	gsl_vector *tau =gsl_vector_alloc(pp);
	gsl_matrix *R =gsl_matrix_alloc(nn,pp);
	gsl_matrix *Q =gsl_matrix_alloc(nn,nn);
	gsl_matrix *R1 =gsl_matrix_alloc(pp,pp);
	gsl_matrix *Q1 =gsl_matrix_alloc(nn,pp);
	gsl_matrix *evec1 =gsl_matrix_alloc(nn,pp);

    for (int m=bott;m<top;m++){
		for (int i = 0; i < nn; i++){
			for (int j = 0; j <pp1; j++)
				gsl_matrix_set(m1s, i, j, gt[i][pp*m+j]);
	    }//m1s=X

		for(int k=0;k<nd/dd;k++){  
			for (int yi = 0; yi < nn; yi++){
				for (int yj = 0; yj < dd; yj++)
			        gsl_matrix_set(m1y, yi, yj, ph[yi][dd*k+yj]);
	         }//m1y=Y

			double lgbf1,lgbf2,lgbf3;
            if(dis>0){               
			    gsl_linalg_QR_decomp (m1s,tau);
				gsl_linalg_QR_unpack (m1s,tau,Q,R);
				for(int i=0;i<pp;i++){
					for(int j=0;j<pp;j++)                 
						gsl_matrix_set(R1,i,j,gsl_matrix_get(R,i,j));    
			    }
			    for(int i=0;i<nn;i++){
					for(int j=0;j<pp;j++)
						gsl_matrix_set(Q1,i,j,gsl_matrix_get(Q,i,j));
			    }
				gsl_blas_dgemm(CblasNoTrans,CblasTrans,1,R1,R1,0,m1ss);//(pp-1)*s=R*t(R)  the scale of (pp-1) will not influence the eigen vectors and the propotions of eigen values
                double x=pp-1;
				gsl_matrix_scale(m1ss,1/x);
				gsl_eigen_symmv (m1ss, eval, evec, w);
			
                gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1,Q1,evec,0,evec1);
            /*    cout<<"test"<<endl;
				for(int a=1;a<10;a++){
					double b=a;
					b=b/10;
                    double c=lgbf_v(m1y,evec1,eval,w,b,0,dd,pp)/log(10);
					cout<<"sa="<<b<<" lgbf="<<c<<endl;
				}
				cout<<"test end "<<endl;       */

	            lgbf1=lgbf_v(m1y,evec1,eval,w,0.1,0,dd,pp);
	            lgbf2=lgbf_v(m1y,evec1,eval,w,0.2,0,dd,pp);
	            lgbf3=lgbf_v(m1y,evec1,eval,w,0.5,0,dd,pp);
			}
			else{
                lgbf1=lgbf_mm(m1s,m1y,0.1,0,dd,pp1);
		        lgbf2=lgbf_mm(m1s,m1y,0.2,0,dd,pp1);
		        lgbf3=lgbf_mm(m1s,m1y,0.5,0,dd,pp1);
			}
	   	    double lgbf=lgbf1;
	        if(lgbf<lgbf2) {lgbf=lgbf2;}
	        if(lgbf<lgbf3){ lgbf=lgbf3;}
	        double lgbfa=log((exp(lgbf1-lgbf)+exp(lgbf2-lgbf)+exp(lgbf3-lgbf))/3);
	        lgbfa=(lgbfa+lgbf)/log(10);

            char buf[100];
	        if(lgbfa<1e-3)
	        sprintf(buf, "%.3f",lgbfa);
	        else sprintf(buf,"%+.3f",lgbfa);
	        outfile <<buf<<"\t";
	    }
       outfile<<endl;
	}
	gsl_matrix_free(m1s);
	gsl_matrix_free(m1y);
	gsl_matrix_free(m1ss);
	gsl_matrix_free(evec);
	gsl_vector_free(eval);
	gsl_eigen_symmv_free(w);
    gsl_vector_free(tau);
	gsl_matrix_free(Q);
	gsl_matrix_free(R);
    gsl_matrix_free(evec1);
	gsl_matrix_free(Q1);
	gsl_matrix_free(R1);
    outfile.close();
	cout << sfn << " has been created." << endl;

	
}

void ModelnData::read_multi_geno_multi_pheno(int l1, int l2, int t){

    ifstream in (vPin.at(0).c_str());
    Mat a;
    istringstream istr;
    string str;
    vector<double> tmpvec;
    while(getline(in,str)){
	   istr.str(str);
	   double tmp;
	   while(istr>>tmp){
		   tmpvec.push_back(tmp);
	   }
       a.push_back(tmpvec);
	   tmpvec.clear() ;
	   istr.clear();
    }
    in.close();
	nd=a[0].size();
	int nny=a.size();
 
	ph = Allocate2DMatrix(nny, nd);
    for (int i = 0; i < nny;i++) {
        for(int j = 0; j < nd;j++){
	    ph[i][j]=a[i][j];
		}
    }

    ifstream ing (vGin.at(0).c_str());
	Mat b;
	istringstream istrg;
	string strg;
    vector<double> tmpvecg;
		while(getline(ing,strg)){
			istrg.str(strg);
		    double tmp;
		    while(istrg>>tmp){
				tmpvecg.push_back(tmp); 
	        }
	        b.push_back(tmpvecg);
	        tmpvecg.clear() ;
	        istrg.clear();
		}
	ing.close();
	int nnx=0;
	if(t==0){
	nnx=b[0].size();
	if(l1==-1&&l2==-1){
    np=b.size();
	}
	else{np=l2-l1+1;}
	}
	if(t==1){
		nnx=b.size();
		if(l1==-1&&l2==-1){
			   np=b[0].size();
		}
		else{np=l2-l1+1;}
	}
	gt = Allocate2DMatrix(nnx, np);
	if(l1==-1&&l2==-1){
		      cout<<"-binbam: reading from the first line to the last line of the genotype file"<<endl;
			if(t==0){
               for (int i = 0; i < np;i++) {
                 for(int j = 0; j < nnx;j++){
	                 gt[j][i]=b[i][j];
		           }     
                 }
             	}
            if(t==1){
     for (int i = 0; i < np;i++) {
		               for(int j = 0; j < nnx;j++){
			gt[j][i]=b[j][i];
										               }
					        }

	}
	} 
	else{
     if(t==0){          
		 cout<<"-binbam: reading from line "<<l1<<" to line "<<l2<<" of the covariates file"<<endl;
		     for (int i = 0; i < np;i++) {
				               for(int j = 0; j < nnx;j++){
								                 gt[j][i]=b[l1-1+i][j];
												               }
							        }
			     }
	     if(t==1){
		   cout<<"-binbam: reading from column "<<l1<<" to column "<<l2<<" of the covariates file"<<endl;
			      for (int i = 0; i < np;i++) {
					                         for(int j = 0; j < nnx;j++){
												             gt[j][i]=b[j][l1-1+i];
															                                                        }
											                             }

				      }

	}

	if(nnx==nny)nn=nnx;
	else{
		cout << "-bimbam: the individual numbers of two files are not the same " << endl;
		safe_exit();
	}
	cout << "-bimbam: the genotype file contains "<<nn<<" individuals and "<<np<<" covariates " << endl;
	cout << "-bimbam: the phenotype file contains "<<nn<<" individuals and "<<nd<<" phenotypes" << endl;
	
	center_col(gt, nn, np);
	center_col(ph, nn, nd);   
}

void ModelnData::end() {

    Free2DMatrix(gt);
	Free2DMatrix(ph);


}






void ModelnData::disormm(int dd, int pp,int dis){
	
    cout << "-bimbam: dd is "<<dd<<" pp is "<<pp<<endl;

    fstream outfile;
    string sfn("output/");
	sfn.append(fnOutput);
	if (dis==0){
		sfn.append(".mm.txt");
    }
	else{
		sfn.append(".dis.txt");
	}
	outfile.open(sfn.c_str(), ios::out);
	if(!outfile.is_open()){
		cout << "can't open file ... " << endl;
		exit(0);
	 }


    gsl_matrix * m1s = gsl_matrix_alloc(nn, pp);
	gsl_matrix * m1y = gsl_matrix_alloc(nn, dd);

	gsl_matrix * m1ss = gsl_matrix_alloc(pp, pp);//1.m1ss=S
	gsl_vector *eval = gsl_vector_alloc (pp);//6
	gsl_matrix *evec = gsl_matrix_alloc (pp, pp);//7
	gsl_eigen_symmv_workspace * w = gsl_eigen_symmv_alloc (pp);
	gsl_vector *tau =gsl_vector_alloc(pp);
	gsl_matrix *R =gsl_matrix_alloc(nn,pp);
	gsl_matrix *Q =gsl_matrix_alloc(nn,nn);
	gsl_matrix *R1 =gsl_matrix_alloc(pp,pp);
	gsl_matrix *Q1 =gsl_matrix_alloc(nn,pp);
	gsl_matrix *evec1 =gsl_matrix_alloc(nn,pp);
	gsl_vector *eval2 = gsl_vector_alloc (nn);//6
	gsl_matrix *evec2 = gsl_matrix_alloc (nn, nn);//7
	gsl_eigen_symmv_workspace * w2 = gsl_eigen_symmv_alloc (nn);

    for (int m=0;m<(np/pp);m++){
		for (int i = 0; i < nn; i++){
			for (int j = 0; j <pp; j++)
				gsl_matrix_set(m1s, i, j, gt[i][pp*m+j]);
	    }//m1s=X
		
		if(dis>0 && pp<nn){  
		
			gsl_linalg_QR_decomp (m1s,tau);
			gsl_linalg_QR_unpack (m1s,tau,Q,R);
			for(int i=0;i<pp;i++){
				for(int j=0;j<pp;j++)                 
					gsl_matrix_set(R1,i,j,gsl_matrix_get(R,i,j));    
			  
			}
			for(int i=0;i<nn;i++){
				for(int j=0;j<pp;j++)
					gsl_matrix_set(Q1,i,j,gsl_matrix_get(Q,i,j));
			}
			gsl_blas_dgemm(CblasNoTrans,CblasTrans,1,R1,R1,0,m1ss);//(pp-1)*s=R*t(R)  the scale of (pp-1) will not influence the eigen vectors and the propotions of eigen values
			double x=pp-1;
			gsl_matrix_scale(m1ss,1/x);
			gsl_eigen_symmv (m1ss, eval, evec, w);
			gsl_eigen_symmv_sort (eval, evec, GSL_EIGEN_SORT_VAL_DESC);
			gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1,Q1,evec,0,evec1);
		}
		if(dis>0 && pp>=nn){

			gsl_blas_dgemm(CblasNoTrans,CblasTrans,1,m1s,m1s,0,Q);
			double x=nn-1;
			gsl_matrix_scale(Q,1/x);
			gsl_eigen_symmv (Q, eval2, evec2, w2);
			gsl_eigen_symmv_sort (eval2, evec2, GSL_EIGEN_SORT_VAL_DESC);
		}
		
		for(int k=0;k<nd/dd;k++){  
			for (int yi = 0; yi < nn; yi++){
				for (int yj = 0; yj < dd; yj++)
			        gsl_matrix_set(m1y, yi, yj, ph[yi][dd*k+yj]);
	         }//m1y=Y
			double lgbfa=0, p=0;
            if(dis>0){  
              
            /*    cout<<"test"<<endl;
				for(int a=1;a<10;a++){
					double b=a;
					b=b/10;
                    double c=/lgbf_v(m1y,evec1,eval,w,b,0,dd,pp)/log(10);
					cout<<"sa="<<b<<" lgbf="<<c<<endl;
				}
				cout<<"test end "<<endl;       */
                if(pp<nn){
				double lgbf1=lgbf_v(m1y,evec1,eval,w,0.1,0,dd,pp);
                                double lgbf2=lgbf_v(m1y,evec1,eval,w,0.2,0,dd,pp);
                                double lgbf3=lgbf_v(m1y,evec1,eval,w,0.5,0,dd,pp);
                                double lgbf=lgbf1;
                                if(lgbf<lgbf2) {lgbf=lgbf2;}
                                if(lgbf<lgbf3){ lgbf=lgbf3;}
                                lgbfa=log((exp((lgbf1-lgbf)*log(10))+exp((lgbf2-lgbf)*log(10))+exp((lgbf3-lgbf)*log(10)))/3);
                                lgbfa=(lgbfa+lgbf*log(10))/log(10);
                               // cout<<"lgbfa="<<lgbfa<<endl;
				}        
				else{                    
                                 lgbfa=lgbf_v(m1y,evec2,eval2,w2,0,0,dd,pp);
				}
			}

			else{ 
				if(opti==0){
                  double lgbf1=lgbf_mm(m1s,m1y,0.1,0,dd,pp);
		          double  lgbf2=lgbf_mm(m1s,m1y,0.2,0,dd,pp);
		          double lgbf3=lgbf_mm(m1s,m1y,0.5,0,dd,pp);
                  double lgbf=lgbf1;
                 if(lgbf<lgbf2) {lgbf=lgbf2;}
			     if(lgbf<lgbf3){ lgbf=lgbf3;}
			     lgbfa=log((exp((lgbf1-lgbf)*log(10))+exp((lgbf2-lgbf)*log(10))+exp((lgbf3-lgbf)*log(10)))/3);
			     lgbfa=(lgbfa+lgbf*log(10))/log(10);
		         }
				if(opti==1){
                lgbfa=lgbf_mm(m1s,m1y,0,0,dd,pp);     
				}
				if (pval==1 && dd==1){
				 p=pvalue_mm(m1s, m1y, dd,pp);
				}
          			}
            char buf[100];
	        if(lgbfa<1e-3)
	        sprintf(buf, "%.3f",lgbfa);
	        else sprintf(buf,"%+.3f",lgbfa);
	        outfile <<buf<<"\t";
			if (pval==1 && dd==1){
            
			sprintf(buf,"%e",p);
			outfile <<buf<<"\t";
			}
	    }
       outfile<<endl;


	}
	gsl_matrix_free(m1s);
	gsl_matrix_free(m1y);
	gsl_matrix_free(m1ss);
	gsl_matrix_free(evec);
	gsl_vector_free(eval);
	gsl_eigen_symmv_free(w);
    gsl_vector_free(tau);
	gsl_matrix_free(Q);
	gsl_matrix_free(R);
    gsl_matrix_free(evec1);
	gsl_matrix_free(Q1);
	gsl_matrix_free(R1);
	gsl_matrix_free(evec2);
	gsl_vector_free(eval2);
	gsl_eigen_symmv_free(w2);
    outfile.close();
	cout << sfn << " has been created." << endl;


}

double ModelnData::pvalue_mm(gsl_matrix *m1s, gsl_matrix *m1y, int dd,int pp){
	int status;
	int iter=0;
	int max_iter=100;
	const gsl_root_fsolver_type *T= gsl_root_fsolver_brent;
	gsl_root_fsolver *s=gsl_root_fsolver_alloc (T);
	double r = 0;
	double x_lo =0.00001, x_hi = 1;
	struct my1_f_params * params = new struct my1_f_params;
	params->y=m1y;
	params->x=m1s;
	params->n=nn;
	params->d=dd;
	params->p=pp;
	double a=my1_f(x_lo,(void *)params);
	double b=my1_f(x_hi,(void *)params);
	int ctrl=1;
	while(b>0 && ctrl==1){
		x_hi=x_hi*10;
		b=my1_f(x_hi,(void *)params);
		if(fabs(b)<0.00000001){
			 r=x_hi;
			 ctrl=0;
		}
	 }
	 while(a<0 && ctrl==1){
		x_lo=x_lo/10;
		a=my1_f(x_lo,(void *)params);
		if(fabs(a)<0.00000001){
			  r=x_lo;
			  ctrl=0;
		}
		if(fabs(x_lo)<0.00000001){
			  r=x_lo;
			  ctrl=0;
		}
	}
	if(a>0 && b<0 && r==0){
		 gsl_function F;
		 F.function = &my1_f;
		 F.params = params;
		 gsl_root_fsolver_set (s, &F, x_lo, x_hi);
		 do{
				 iter++;
				 status = gsl_root_fsolver_iterate (s);
				 r = gsl_root_fsolver_root (s);
				 x_lo = gsl_root_fsolver_x_lower (s);
				 x_hi = gsl_root_fsolver_x_upper (s);
				 status = gsl_root_test_interval (x_lo, x_hi,0, 0.001);
		 }
		 while (status == GSL_CONTINUE && iter<max_iter);
		 gsl_root_fsolver_free (s);
	}
	
	delete params;
	gsl_matrix *B=gsl_matrix_alloc(pp,dd);//dd=1
	gsl_matrix *tmp1=gsl_matrix_alloc(pp,pp);
    gsl_blas_dgemm(CblasTrans,CblasNoTrans,1,m1s,m1s,0,tmp1);
	gsl_matrix *tmp2=gsl_matrix_alloc(pp,pp);
	gsl_matrix_set_identity(tmp2);
    gsl_matrix_scale(tmp2,(1/r));
	gsl_matrix_add(tmp1,tmp2);
	GetInverse(tmp1,tmp2);
	gsl_matrix *tmp3=gsl_matrix_alloc(pp,nn);
	gsl_blas_dgemm(CblasNoTrans,CblasTrans,1,tmp2,m1s,0,tmp3);
    gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1,tmp3,m1y,0,B);
	gsl_matrix *Y=gsl_matrix_alloc(nn,dd);//dd=1
	gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1,m1s,B,0,Y);
	double c=0, d=0;
	for(int i=0;i<nn;i++){
       c=c+(gsl_matrix_get(Y,i,0)*gsl_matrix_get(Y,i,0));
	   d=d+(gsl_matrix_get(m1y,i,0)-gsl_matrix_get(Y,i,0))*(gsl_matrix_get(m1y,i,0)-gsl_matrix_get(Y,i,0));

	}
	double f=c/d*(nn-pp-1)/pp;
//	cout<<"f="<<f<<endl;
        f=1-gsl_cdf_fdist_P (f, pp, (nn-pp-1));
        f=-log(f)/log(10);
//	cout<<f<<endl;
	return f;
	gsl_matrix_free(B);
	gsl_matrix_free(Y);
	gsl_matrix_free(tmp1);
	gsl_matrix_free(tmp2);
	gsl_matrix_free(tmp3);

}


//add bddhanlix end.                      
