
#include <sstream> 
#include <iostream> 
#include <fstream> 
#include <vector> 
#include <algorithm>
#include <map>
#include <set> 
#include <stdio.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_permutation.h>
#include "gsl/gsl_rng.h"
#include "gsl/gsl_multimin.h"
#include "gsl/gsl_blas.h"
#include "gsl/gsl_linalg.h"
#include "gsl/gsl_randist.h"
#include "gsl/gsl_permute.h"
#include "gsl/gsl_sf_gamma.h"
#include "string.h"
#include "fpmath.h"
     
using namespace std; 

//global gsl variable in use; 
const gsl_rng_type * gslType;
gsl_rng * gsl_r; 

#define com_file 1
#define com_seed 2
#define com_dim 3
#define com_step 4
#define com_warm 5
#define com_out 6
#define com_chain 7
#define com_temp 8
#define com_help 9
#define com_expfile 10
#define com_sw 11
int main(int argc, char ** argv)
{
	string fin; 
	string fout; 
	string fexp; 
	double sw = 0.3; 
	int dim = 0; 
	int nSteps = 0;  
	int burnin = 0; 
	int nchains = 1; 
	double ladder = 1; 
	unsigned long seed = time(NULL); 

	map<string, int> hcom; 
	hcom["-in"] = com_file; 
	hcom["-exp"] = com_expfile; 
	hcom["-out"] = com_out; 
	hcom["-r"] = com_seed; 
	hcom["-d"] = com_dim; 
	hcom["-s"] = com_step; 
	hcom["-w"] = com_warm; 
	hcom["-c"] = com_chain; 
	hcom["-t"] = com_temp; 
	hcom["-h"] = com_help; 
	hcom["-sw"] = com_sw; 
	
	for(int i = 1; i < argc; i++) 
	{   
		string str;  
		string opt; 
		if(argv[i][0] != '-') 
			continue;
		
		str.assign(argv[i]);
		opt.assign(str, 0, str.length());

		map<string, int> :: iterator iter; 
		iter = hcom.find(opt); 
		if(iter == hcom.end())
		{
			cout << "unknown option: " << opt << endl; 
			exit(0); 
		}
		
		switch (hcom[opt]) 
		{			
			case com_help: 
				cout << "-in, -out, -exp, -rdswcth" << endl; 
				cout << "r : random seed" << endl; 
				cout << "d : dimension (number of nodes)" << endl; 
				cout << "s : mcmc steps" << endl; 
				cout << "w : mcmc burn in steps" << endl; 
				cout << "c : number of chains" << endl; 
				cout << "t : temperature ladder" << endl; 
				cout << "h : print this help" << endl; 
				exit(0); 

			case com_file:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				fin.clear();
				fin.assign(argv[i+1]);
				break;
				
			case com_expfile:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				fexp.clear();
				fexp.assign(argv[i+1]);
				break;
				
			case com_out:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				fout.clear();
				fout.assign(argv[i+1]);
				break;
				
			case com_seed: // rand seed 
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				if(!isdigit(argv[i+1][0]))
				{
					cout << "wrong argument after option." << endl; 
					exit(0); 
				}
				seed = (unsigned long) atoi(argv[i+1]);
				break;
				
            case com_dim:
                if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
                if(!isdigit(argv[i+1][0]))
                {
                    cout << "wrong augument after option." << endl;
                    exit(0);
                }
                dim = atoi(argv[i+1]);
                break;
				
			case com_step:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				if(!isdigit(argv[i+1][0]))
				{
					cout << "wrong argument after option." << endl; 
					exit(0); 
				}
				nSteps = atoi(argv[i+1]);
				break;
				
			case com_chain:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				if(!isdigit(argv[i+1][0]))
				{
					cout << "wrong argument after option." << endl; 
					exit(0); 
				}
				nchains = atoi(argv[i+1]);
				break;
				
			case com_temp:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				if(!isdigit(argv[i+1][0]))
				{
					cout << "wrong argument after option." << endl; 
					exit(0); 
				}
				ladder = (double) atof(argv[i+1]);
				break;
				
			case com_sw:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				if(!isdigit(argv[i+1][0]))
				{
					cout << "wrong argument after option." << endl; 
					exit(0); 
				}
				sw = (double) atof(argv[i+1]);
				break;

			case com_warm:
				if(argv[i+1] == NULL || argv[i+1][0] == '-') continue;
				if(!isdigit(argv[i+1][0]))
				{
					cout << "wrong argument after option." << endl; 
					exit(0); 
				}
				burnin = atoi(argv[i+1]);
				break;

			default:
				fprintf(stderr,"Bad option %s\n", argv[i]);
				exit(0);
				break; 
		}              
	}                          
	
	gsl_rng_env_setup();
	gslType = gsl_rng_default;
	gsl_r = gsl_rng_alloc (gslType);
	gsl_rng_set(gsl_r, seed);
	cout << "random seed = " << seed << endl; 
	//set up random number generator; 

	fstream outfile; 
	outfile.open(fout.c_str(), ios::out);
	if(!outfile.is_open()) 
	{
		cout << "can't open file to write ... " << endl; 
		exit(0); 
	}

//	outfile << " ### "; 
//	for (int i = 0; i < argc; i++)
//		outfile << argv[i] << " "; 
//	outfile << endl; 


	double ** cov = NULL; 
	int nc = 0; 
	int ni = 0; 
	{
		ifstream in; 
		in.open(fin.c_str());
		vector<vector<double> > b;
		istringstream istr;
		string str;
		vector<double> tmpvec;
		while(getline(in,str)){
		   istr.str(str);
		   double tmp;
		   while(istr>>tmp){
			   tmpvec.push_back(tmp);
		   }
		   b.push_back(tmpvec);
		   tmpvec.clear() ;
		   istr.clear();
		}
		in.close();
		ni = (int) b.size(); 
		nc = b.at(0).size(); 
		cov = Allocate2DMatrix(ni, nc);  
		for (int i = 0; i < ni; i++)
			for (int j = 0; j < nc; j++)
				cov[i][j] = b.at(i).at(j); 
		vector<vector<double> > ().swap(b); 
	}
	cout << ni << endl; 
	cout << nc << endl; 
//	for (int i = 0; i < ni; i++)
//	{
//		for (int j = 0; j < nc; j++)
//			cout << cov[i][j] << " "; 
//		cout << endl; 
//	}
    
	for (int j = 0; j < nc; j++)
	{
		double sum1 = 0; 
		double sum2 = 0; 
		int count = 0; 
		for (int i = 0; i < ni; i++)
		{
			if(cov[i][j] != 9) {
				count++; 
				sum1 += cov[i][j]; 
				sum2 += cov[i][j] * cov[i][j]; 
			}
		}
		sum1 /= count; 
		sum2 /= count; 
		sum2 -= sum1 * sum1; 
		sum2 = sqrt(sum2); 

//		cout << sum1 << endl; 
//		cout << sum2 << endl; 
		for (int i = 0; i < ni; i++)
		{
			if(cov[i][j] == 9) {
				cov[i][j] = 0;  
			}
			else 
			{
				cov[i][j] -= sum1; 
				if(sum2 > 1e-10)
					cov[i][j] /= sum2; 
			}

		}

	}

//	for (int i = 0; i < ni; i++)
//	{
//		for (int j = 0; j < nc; j++)
//			cout << cov[i][j] << " "; 
//		cout << endl; 
//	}
    
	double ** xtx = Allocate2DMatrix(ni, ni); 
	for (int i = 0; i < ni; i++)
		for (int j = 0; j <= i; j++)
		{
			xtx[i][j] = 0; 
			for (int k = 0; k < nc; k++)
				xtx[i][j] += cov[i][k] * cov[j][k]; 
			xtx[j][i] = xtx[i][j]; 
		}

	for (int i = 0; i < ni; i++)
	{
		for (int j = 0; j < ni; j++)
			outfile << xtx[i][j] << " "; 
		outfile << endl; 
	}
	outfile.close(); 

	Free2DMatrix(xtx); 
	Free2DMatrix(cov); 


}
